@@ -1,7 +1,10 @@
 """Miscellaneous functions used across the library that does not fit into other packages."""
 
+from __future__ import annotations
 import builtins
 
+import numpy as np
+
 from mydatapreprocessing import preprocessing
 
 # Lazy imports
@@ -10,15 +13,15 @@ from mydatapreprocessing import preprocessing
 
 class Global_vars:
     def __init__(self) -> None:
-        self._JUPYTER = True if hasattr(builtins, "__IPYTHON__") else 0
-        self._GUI = False
-        self._PLOTS_CONFIGURED = False
+        self.JUPYTER: bool = True if hasattr(builtins, "__IPYTHON__") else False
+        self.GUI: bool = False
+        self.PLOTS_CONFIGURED: bool = False
 
 
 GLOBAL_VARS = Global_vars()
 
 
-def setup_plots():
+def setup_plots() -> None:
     from pandas.plotting import register_matplotlib_converters
 
     register_matplotlib_converters()
@@ -26,15 +29,17 @@ def setup_plots():
     try:
         from IPython import get_ipython
 
-        if GLOBAL_VARS._JUPYTER:
+        if GLOBAL_VARS.JUPYTER:
             get_ipython().run_line_magic("matplotlib", "inline")
     except Exception:
         pass
 
-    GLOBAL_VARS._CONFIGURED = True
+    GLOBAL_VARS.PLOTS_CONFIGURED = True
 
 
-def confidence_interval(data, predicts=7, confidence=0.1, p=1, d=0, q=0):
+def confidence_interval(
+    data: np.ndarray, predicts: int = 7, confidence: float = 0.1, p: int = 1, d: int = 0, q: int = 0
+):
     """Function to find confidence interval of prediction for graph.
 
     Args:
